package proxy.util;

import com.google.common.collect.Lists;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonArray;
import com.google.gson.JsonDeserializationContext;
import com.google.gson.JsonDeserializer;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParseException;
import com.google.gson.JsonPrimitive;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;
import java.lang.reflect.Type;
import java.util.List;
import java.util.Map.Entry;

public abstract class ChatComponent
{
	private List<ChatComponent> siblings = Lists.<ChatComponent>newArrayList();
	private ChatStyle style;

    public static class Serializer implements JsonDeserializer<ChatComponent>, JsonSerializer<ChatComponent>
    {
        private static final Gson GSON;

        public ChatComponent deserialize(JsonElement p_deserialize_1_, Type p_deserialize_2_, JsonDeserializationContext p_deserialize_3_) throws JsonParseException
        {
            if (p_deserialize_1_.isJsonPrimitive())
            {
                return new ChatComponentText(p_deserialize_1_.getAsString());
            }
            else if (!p_deserialize_1_.isJsonObject())
            {
                if (p_deserialize_1_.isJsonArray())
                {
                    JsonArray jsonarray1 = p_deserialize_1_.getAsJsonArray();
                    ChatComponent ichatcomponent1 = null;

                    for (JsonElement jsonelement : jsonarray1)
                    {
                        ChatComponent ichatcomponent2 = this.deserialize(jsonelement, jsonelement.getClass(), p_deserialize_3_);

                        if (ichatcomponent1 == null)
                        {
                            ichatcomponent1 = ichatcomponent2;
                        }
                        else
                        {
                            ichatcomponent1.siblings.add(ichatcomponent2);
                        }
                    }

                    return ichatcomponent1;
                }
                else
                {
                    throw new JsonParseException("Don\'t know how to turn " + p_deserialize_1_.toString() + " into a Component");
                }
            }
            else
            {
                JsonObject jsonobject = p_deserialize_1_.getAsJsonObject();
                ChatComponent ichatcomponent;

                if (jsonobject.has("text"))
                {
                    ichatcomponent = new ChatComponentText(jsonobject.get("text").getAsString());
                }
                else if (jsonobject.has("translate"))
                {
                    String s = jsonobject.get("translate").getAsString();

                    if (jsonobject.has("with"))
                    {
                        JsonArray jsonarray = jsonobject.getAsJsonArray("with");
                        Object[] aobject = new Object[jsonarray.size()];

                        for (int i = 0; i < aobject.length; ++i)
                        {
                            aobject[i] = this.deserialize(jsonarray.get(i), p_deserialize_2_, p_deserialize_3_);

                            if (aobject[i] instanceof ChatComponentText)
                            {
                                ChatComponentText chatcomponenttext = (ChatComponentText)aobject[i];

                                if ((((ChatComponent)chatcomponenttext).style == null || ((ChatComponent)chatcomponenttext).style.isEmpty()) && ((ChatComponent)chatcomponenttext).siblings.isEmpty())
                                {
                                    aobject[i] = chatcomponenttext.getText();
                                }
                            }
                        }

                        ichatcomponent = new ChatComponentTranslation(s, aobject);
                    }
                    else
                    {
                        ichatcomponent = new ChatComponentTranslation(s, new Object[0]);
                    }
                }
                else if (jsonobject.has("score"))
                {
                    JsonObject jsonobject1 = jsonobject.getAsJsonObject("score");

                    if (!jsonobject1.has("name") || !jsonobject1.has("objective"))
                    {
                        throw new JsonParseException("A score component needs a least a name and an objective");
                    }

                    ichatcomponent = new ChatComponentScore(JsonUtils.getString(jsonobject1, "name"), JsonUtils.getString(jsonobject1, "objective"));

                    if (jsonobject1.has("value"))
                    {
                        ((ChatComponentScore)ichatcomponent).setValue(JsonUtils.getString(jsonobject1, "value"));
                    }
                }
                else
                {
                    if (!jsonobject.has("selector"))
                    {
                        throw new JsonParseException("Don\'t know how to turn " + p_deserialize_1_.toString() + " into a Component");
                    }

                    ichatcomponent = new ChatComponentSelector(JsonUtils.getString(jsonobject, "selector"));
                }

                if (jsonobject.has("extra"))
                {
                    JsonArray jsonarray2 = jsonobject.getAsJsonArray("extra");

                    if (jsonarray2.size() <= 0)
                    {
                        throw new JsonParseException("Unexpected empty array of components");
                    }

                    for (int j = 0; j < jsonarray2.size(); ++j)
                    {
                        ichatcomponent.siblings.add(this.deserialize(jsonarray2.get(j), p_deserialize_2_, p_deserialize_3_));
                    }
                }

                ichatcomponent.style = (ChatStyle)p_deserialize_3_.deserialize(p_deserialize_1_, ChatStyle.class);
                return ichatcomponent;
            }
        }

        private void serializeChatStyle(ChatStyle style, JsonObject object, JsonSerializationContext ctx)
        {
            JsonElement jsonelement = ctx.serialize(style);

            if (jsonelement.isJsonObject())
            {
                JsonObject jsonobject = (JsonObject)jsonelement;

                for (Entry<String, JsonElement> entry : jsonobject.entrySet())
                {
                    object.add((String)entry.getKey(), (JsonElement)entry.getValue());
                }
            }
        }

        public JsonElement serialize(ChatComponent p_serialize_1_, Type p_serialize_2_, JsonSerializationContext p_serialize_3_)
        {
            if (p_serialize_1_ instanceof ChatComponentText && (p_serialize_1_.style == null || p_serialize_1_.style.isEmpty()) && p_serialize_1_.siblings.isEmpty())
            {
                return new JsonPrimitive(((ChatComponentText)p_serialize_1_).getText());
            }
            else
            {
                JsonObject jsonobject = new JsonObject();

                if (p_serialize_1_.style != null && !p_serialize_1_.style.isEmpty())
                {
                    this.serializeChatStyle(p_serialize_1_.style, jsonobject, p_serialize_3_);
                }

                if (!p_serialize_1_.siblings.isEmpty())
                {
                    JsonArray jsonarray = new JsonArray();

                    for (ChatComponent ichatcomponent : p_serialize_1_.siblings)
                    {
                        jsonarray.add(this.serialize((ChatComponent)ichatcomponent, ichatcomponent.getClass(), p_serialize_3_));
                    }

                    jsonobject.add("extra", jsonarray);
                }

                if (p_serialize_1_ instanceof ChatComponentText)
                {
                    jsonobject.addProperty("text", ((ChatComponentText)p_serialize_1_).getText());
                }
                else if (p_serialize_1_ instanceof ChatComponentTranslation)
                {
                    ChatComponentTranslation chatcomponenttranslation = (ChatComponentTranslation)p_serialize_1_;
                    jsonobject.addProperty("translate", chatcomponenttranslation.getKey());

                    if (chatcomponenttranslation.getFormatArgs() != null && chatcomponenttranslation.getFormatArgs().length > 0)
                    {
                        JsonArray jsonarray1 = new JsonArray();

                        for (Object object : chatcomponenttranslation.getFormatArgs())
                        {
                            if (object instanceof ChatComponent)
                            {
                                jsonarray1.add(this.serialize((ChatComponent)((ChatComponent)object), object.getClass(), p_serialize_3_));
                            }
                            else
                            {
                                jsonarray1.add(new JsonPrimitive(String.valueOf(object)));
                            }
                        }

                        jsonobject.add("with", jsonarray1);
                    }
                }
                else if (p_serialize_1_ instanceof ChatComponentScore)
                {
                    ChatComponentScore chatcomponentscore = (ChatComponentScore)p_serialize_1_;
                    JsonObject jsonobject1 = new JsonObject();
                    jsonobject1.addProperty("name", chatcomponentscore.getName());
                    jsonobject1.addProperty("objective", chatcomponentscore.getObjective());
                    jsonobject1.addProperty("value", chatcomponentscore.getValue());
                    jsonobject.add("score", jsonobject1);
                }
                else
                {
                    if (!(p_serialize_1_ instanceof ChatComponentSelector))
                    {
                        throw new IllegalArgumentException("Don\'t know how to serialize " + p_serialize_1_ + " as a Component");
                    }

                    ChatComponentSelector chatcomponentselector = (ChatComponentSelector)p_serialize_1_;
                    jsonobject.addProperty("selector", chatcomponentselector.getSelector());
                }

                return jsonobject;
            }
        }

        public static String componentToJson(ChatComponent component)
        {
            return GSON.toJson((Object)component);
        }

        public static ChatComponent jsonToComponent(String json)
        {
            return (ChatComponent)GSON.fromJson(json, ChatComponent.class);
        }

        static
        {
            GsonBuilder gsonbuilder = new GsonBuilder();
            gsonbuilder.registerTypeHierarchyAdapter(ChatComponent.class, new ChatComponent.Serializer());
            gsonbuilder.registerTypeHierarchyAdapter(ChatStyle.class, new ChatStyle.Serializer());
            gsonbuilder.registerTypeAdapterFactory(new EnumTypeAdapterFactory());
            GSON = gsonbuilder.create();
        }
    }
}
